Summary: The Linux kernel (the core of the Linux operating system)

# What parts do we want to build?  We must build at least one kernel.
# These are the kernels that are built IF the architecture allows it.

%define buildup 1
##%define buildsmp 1
%define buildsmp 0
%define buildbigmem 0
%define buildsource 1


# Versions of various parts

#
# Polite request for people who spin their own kernel rpms:
# please modify the "release" field in a way that identifies
# that the kernel isn't the stock RHEL kernel, for example by
# adding some text to the end of the version number.
#
%define builddevfs 1
%define sublevel 0
%define kversion 2.6.%{sublevel}
%define rpmversion 2.6.%{sublevel}
#%define release 0.%(R="$Revision: 1.14 $"; RR="${R##: }"; echo ${RR%%?})
%define devfsdver 1.3.25
%define release b1

%define KVERREL %{PACKAGE_VERSION}-%{PACKAGE_RELEASE}

# groups of related archs
%define all_x86 i386 i586 i686

# Override generic defaults with per-arch defaults

# First, architecture-specific kernels off on all other archs (ifnarch)
%ifnarch i686
%define buildbigmem 0
%endif
##%ifnarch i386 x86_64
##%define buildsource 0
##%endif

# Second, per-architecture exclusions (ifarch)
%ifarch i386
%define buildsmp 0
%define buildup 0
%endif

%ifarch %{all_x86}
%define all_arch_configs kernel-%{kversion}-i?86*.config
%define _64bit 0
%endif

%ifarch x86_64
%define all_arch_configs kernel-%{kversion}-x86_64*.config
%define _64bit 1
%endif

#
# Three sets of minimum package version requirements in the form of Conflicts:
# to versions below the minimum
#

#
# First the general kernel 2.6 required versions as per
# Documentation/Changes
#
%define kernel_dot_org_conflicts  ppp <= 2.3.15, pcmcia-cs <= 3.1.20, isdn4k-utils <= 3.0, mount < 2.10r-5, nfs-utils < 1.0.3, e2fsprogs < 1.29, util-linux < 2.10, jfsutils < 1.0.14, reiserfsprogs < 3.6.3, xfsprogs < 2.1.0, procps < 2.0.9, oprofile < 0.5.3

# 
# Then a series of requirements that are Red Hat specific, either 
# because we add patches for something, or the older versions have 
# problems with the newer kernel or lack certain things that make 
# integration in the distro harder than needed.
#
%define package_conflicts  cipe < 1.4.5, tux < 2.1.0, kudzu <= 0.92, initscripts < 7.23, dev < 3.2-7, iptables < 1.2.5-3, bcm5820 < 1.81, nvidia-rh72 <= 1.0

#
# Several packages had bugs in them that became obvious when the NPTL
# threading code got integrated. 
#
%define nptl_conflicts SysVinit < 2.84-13, pam < 0.75-48, vixie-cron < 3.0.1-73, privoxy < 3.0.0-8, spamassassin < 2.44-4.8.x,  cups < 1.1.17-13

#
# Packages that need to be installed before the kernel is, because the %post
# scripts use them.
#
%define kernel_prereq  fileutils, module-init-tools, initscripts >= 5.83, mkinitrd >= 3.5.5

Name: kernel
Group: System Environment/Kernel
License: GPLv2
Version: %{rpmversion}
Release: %{release}
ExclusiveArch: %{all_x86} x86_64
ExclusiveOS: Linux
Provides: kernel = %{version}
Provides: kernel-drm = 4.3.0
Prereq: %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}
Conflicts: %{nptl_conflicts}
# We can't let RPM do the dependencies automatic because it'll then pick up
# a correct but undesirable perl dependency from the module headers which
# isn't required for the kernel proper to function
AutoReqProv: no

BuildArchitectures: i586


#
# List the packages used during the kernel build
#
BuildPreReq: module-init-tools, patch >= 2.5.4, bash >= 2.03, sh-utils, tar
#BuildPreReq: bzip2, findutils, gzip, m4, perl, make >= 3.78, gnupg
BuildPreReq: bzip2, findutils, gzip, m4, perl, make >= 3.78
BuildConflicts: rhbuildsys(DiskFree) < 500Mb
#BuildRequires: gcc >= 2.96-98, binutils >= 2.12, redhat-rpm-config
BuildRequires: gcc >= 2.96-98, binutils >= 2.12

Source0: ftp://ftp.kernel.org/pub/linux/kernel/v2.6/linux-%{kversion}.tar.bz2
Source1: ftp://ftp.atnf.csiro.au/pub/people/rgooch/linux/daemons/devfsd-v%{devfsdver}.tar.bz2

Source10: COPYING.modules
##Source11: linux-modules-unsupported
##Source12: Makefile.config
##Source13: merge.pl
##Source14: Makefile.configdefs

Source20: config.berry
##Source20: config-generic
##Source21: config-x86-generic
##Source22: config-i586
##Source23: config-i586-smp
##Source24: config-i686
##Source25: config-i686-smp
##Source26: config-i686-bigmem
##Source27: config-x86_64
##Source28: config-x86_64-smp
##Source29: config-x86_64-generic

#
# Patches 0 through 100 are meant for core subsystem upgrades
#
#Patch1: patch-2.6.1-pre1.bz2


#
# Patches 500 through 1000 are reserved for bugfixes to the core system
# and patches related to how RPMs are build
#

Patch500: linux-2.4.0-nonintconfig.patch
Patch510: linux-2.6.0-must_check.patch
Patch520: linux-2.6.0-exec-shield.patch
Patch530: linux-2.6.0-dbus.patch
Patch540: linux-2.6.0-online-resize.patch

#
# Patches 1000 to 5000 are reserved for bugfixes to drivers and filesystems
#

Patch1000: linux-2.4.0-test11-vidfail.patch

#
# 10000 to 20000 is for stuff that has to come last due to the
# amount of drivers they touch. But only theese should go here. 
# Not patches you're too lazy for to put in the proper place.
#

Patch10000: linux-2.6.0-compile.patch
Patch10010: linux-2.6.0-module-license.patch

# END OF PATCH DEFINITIONS

BuildRoot: %{_tmppath}/kernel-%{KVERREL}-root


%package source
Summary: The source code for the Linux kernel.
Group: Development/System
Prereq: fileutils
Requires: make >= 3.78
Requires: gcc >= 3.2
Requires: /usr/bin/strip

%package doc
Summary: Various documentation bits found in the kernel source.
Group: Documentation

%description 
The kernel package contains the Linux kernel (vmlinuz), the core of your
Red Hat Linux operating system.  The kernel handles the basic functions
of the operating system:  memory allocation, process allocation, device
input and output, etc.

%description source
The kernel-source package contains the source code files for the Linux
kernel. The source files can be used to build a custom kernel that is better
tuned to your particular hardware, if you are so inclined (and you know what
you're doing). The customisation guide in the documentation describes in
detail how to do this. This package is not needed nor usable for building
external kernel modules for linking into the default kernel.

%description doc
This package contains documentation files from the kernel
source. Various bits of information about the Linux kernel and the
device drivers shipped with it are documented in these files. 

You'll want to install this package if you need a reference to the
options that can be passed to Linux kernel modules at load time.

%package unsupported
Summary: An extra, unsupported, set of kernel modules
Group: System Environment/Kernel

%description unsupported
This packages contains extra modules for drivers that are in the kernel.org
kernel but that Red Hat can't test properly, but provides as courtesy for
people who want to use these regardless.

%package smp-unsupported
Summary: An extra, unsupported, set of kernel modules
Group: System Environment/Kernel

%description smp-unsupported
This packages contains extra modules for drivers that are in the kernel.org
kernel but that Red Hat can't test properly, but provides as courtesy for
people who want to use these regardless.


%package bigmem-unsupported
Summary: An extra, unsupported, set of kernel modules
Group: System Environment/Kernel

%description bigmem-unsupported
This packages contains extra modules for drivers that are in the kernel.org
kernel but that Red Hat can't test properly, but provides as courtesy for
people who want to use these regardless.

%package smp
Summary: The Linux kernel compiled for SMP machines.
Group: System Environment/Kernel
Provides: kernel = %{version}
Provides: kernel-drm = 4.3.0
Prereq: %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}
Conflicts: %{nptl_conflicts}

%description smp
This package includes a SMP version of the Linux kernel. It is
required only on machines with two or more CPUs as well as machines with
hyperthreading technology.

Install the kernel-smp package if your machine uses two or more CPUs.

%package bigmem
Summary: The Linux Kernel for machines with more than 4 Gigabyte of memory.
Group: System Environment/Kernel
Provides: kernel = %{version}
Provides: kernel-drm = 4.3.0
Prereq: %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}
Conflicts: %{nptl_conflicts}

# upto and including kernel 2.4.9 rpms, the 4Gb+ kernel was called kernel-enterprise
Obsoletes: kernel-enterprise < 2.4.10


%description bigmem
This package includes a Linux kernel that has appropriate configuration options
enabled for Pentium III machines with 4 Gigabyte of memory or more.


%if %{builddevfs}
%package -n devfsd
Summary: Device filesystem management daemon
Group: System Environment/Kernel
Requires: kernel >= 2.3.46

%description -n devfsd
Daemon for managing the device filesystem (devfs)
%endif


###
### prep
###
%prep

%setup -q -n %{name}-%{version} -c -a 1
cd linux-%{kversion}
%if %{builddevfs}
mv ../devfsd* .
%endif

#
# Patches 0 through 100 are meant for core subsystem upgrades
# 

# update to recent upstream bk snapshot

# %patch1 -p1

#
# Patches 500 through 1000 are reserved for bugfixes to the core system
# and patches related to how RPMs are build
#


# This patch adds a "make oldconfig_nonint" which is non-interactive and
# also gives a list of missing options at the end. Useful for automated
# builds (as used in the Red Hat buildsystem).
%patch500 -p1

#
# Patch that adds a __must_check attribute for functions for which checking
# the return value is mantadory (eg copy_from_user)

%patch510 -p1

#
# Exec shield
#
%patch520 -p1

#
# make the kernel spit out DBUS notifications over netlink
#
%patch530 -p1

#
# EXT3 online resize patch (experimental)
#
%patch540 -p1

#
# Patches 1000 to 5000 are reserved for bugfixes to drivers and filesystems
#


# add vidfail capability; 
# without this patch specifying a framebuffer on the kernel prompt would
# make the boot stop if there's no supported framebuffer device; this is bad
# for the installer cd that wants to automatically fall back to textmode
# in that case
%patch1000 -p1

#
# Patches 5000 to 6000 are reserved for new drivers
#


#
# final stuff
#

#
# misc small stuff to make things compile or otherwise improve performance
#
%patch10000 -p1

#
# Add MODULE_LICENSE() where it's missing
#
%patch10010 -p1


# END OF PATCH APPLICATIONS

cp %{SOURCE10} Documentation/

mkdir configs
##cd configs
##cp $RPM_SOURCE_DIR/merge.pl .
##cp $RPM_SOURCE_DIR/Makefile.config .
##cp $RPM_SOURCE_DIR/Makefile.configdefs .
##cp $RPM_SOURCE_DIR/config-* .
##make -f Makefile.config configs
##cd ..
cp $RPM_SOURCE_DIR/config.berry configs/kernel-2.6.0-i586.config
cp -fv configs/%{all_arch_configs} .

# now run oldconfig over all the config files
for i in *.config
do
	mv $i .config
	make nonint_oldconfig > /dev/null
	mv .config configs/$i
done

# make sure the kernel has the sublevel we know it has. This looks weird
# but for -pre and -rc versions we need it since we only want to use
# the higher version when the final kernel is released.
perl -p -i -e "s/^SUBLEVEL.*/SUBLEVEL = %{sublevel}/" Makefile
perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -prep/" Makefile

# get rid of unwanted files resulting from patch fuzz
find . -name "*.orig" -exec rm -fv {} \;
find . -name "*~" -exec rm -fv {} \;


###
### build
###
%build

cd linux-%{kversion}

BuildKernel() {

    # Pick the right config file for the kernel we're building
    if [ -n "$1" ] ; then
	Config=kernel-%{kversion}-%{_target_cpu}-$1.config
    else
	Config=kernel-%{kversion}-%{_target_cpu}.config
    fi

    KernelVer=%{version}-%{release}$1
    echo BUILDING A KERNEL FOR $1 %{_target_cpu}...

    # make sure EXTRAVERSION says what we want it to say
    perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -%{release}$1/" Makefile

    # and now to start the build process

    # init
    make -s mrproper
    cp configs/$Config .config

    # make Makefile and so on... linux-2.4.0-nonintconfig.patch
    make -s nonint_oldconfig > /dev/null
    make -s include/linux/version.h 

    make -s %{?_smp_mflags} bzImage 
    make -s %{?_smp_mflags} modules || exit 1
    
    # Start installing the results

    mkdir -p $RPM_BUILD_ROOT/usr/lib/debug/boot
    install -m 644 System.map $RPM_BUILD_ROOT/usr/lib/debug/boot/System.map-$KernelVer
%if %{_64bit}
    cat System.map  | grep "r __ksymtab" | cut -c30- > exported
%else
    cat System.map  | grep "r __ksymtab" | cut -c22- > exported
%endif
    for i in `cat exported` ; do grep "$i\$" System.map ; done > $RPM_BUILD_ROOT/boot/System.map-$KernelVer
    rm -f exported
    install -m 644 .config $RPM_BUILD_ROOT/boot/config-$KernelVer
    cp arch/*/boot/bzImage $RPM_BUILD_ROOT/boot/vmlinuz-$KernelVer

    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer
    make -s INSTALL_MOD_PATH=$RPM_BUILD_ROOT modules_install KERNELRELEASE=$KernelVer
 
    # And save the headers/makefiles etc for building modules against
    #
    # This all looks scary, but the end result is supposed to be:
    # * all arch relevant include/ files
    # * all Makefile/Kconfig files
    # * all script/ files 

    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    # first copy everything
    cp --parents `find  -type f -name Makefile -o -name "Kconfig*"` $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    # then drop all but the needed Makefiles/Kconfig files
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/Documentation
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    ##cp arch/%{_arch}/kernel/asm-offsets.s $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/%{_arch}/kernel || :
    cp arch/i386/kernel/asm-offsets.s $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/i386/kernel || :
    cp .config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    cp -a scripts $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*.o
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*/*.o
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cd include
    ##cp -a acpi config linux math-emu media net pcmcia rxrpc scsi sound video asm asm-generic asm-%{_arch} $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cp -a acpi config linux math-emu media net pcmcia rxrpc scsi sound video asm asm-generic asm-i386 $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cd ..

    #
    # save the vmlinux file for kernel debugging into the kernel-debuginfo rpm
    #
    mkdir -p $RPM_BUILD_ROOT/usr/lib/debug/lib/modules/$KernelVer
    cp vmlinux $RPM_BUILD_ROOT/usr/lib/debug/lib/modules/$KernelVer
    ln -s /usr/lib/debug/lib/modules/$KernelVer/vmlinux $RPM_BUILD_ROOT/boot/vmlinux-$KernelVer

    # move the modules in the SOURCE11 file to the unsupported/ directory so
    # that they end up in the -unsupported rpms
##    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/unsupported
##    pushd $RPM_BUILD_ROOT/lib/modules/$KernelVer/kernel ; {
##	 tar cf - `cat %{SOURCE11}`  | tar xf - -C ../unsupported
##	 rm -rf `cat %{SOURCE11}`
##    } ; popd


    # mark modules executable so that strip-to-file can strip them
    find $RPM_BUILD_ROOT/lib/modules/$KernelVer -name "*.ko" -type f | xargs chmod u+x

    # detect missing or incorrect license tags
    ##for i in `find $RPM_BUILD_ROOT/lib/modules/$KernelVer -name "*.ko" ` ; do echo -n "$i " ; /sbin/modinfo -l $i ; done | grep -v "\"GPL\"" | grep -v "\"Dual BSD/GPL\"" | grep -v "\"Dual MPL/GPL\"" | grep -v "\"GPL and additional rights\"" | grep -v "\"GPL v2\"" && exit 1
    # remove files that will be auto generated by depmod at rpm -i time
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/modules.*

}

###
# DO it...
###

# prepare directories
rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT/boot

%if %{buildup}
BuildKernel
%endif

%if %{buildsmp}
BuildKernel smp
%endif

%if %{buildbigmem}
BuildKernel bigmem
%endif

%if %{builddevfs}
# compile devfsd
cd devfsd
perl -pi -e "s,-O2,$RPM_OPT_FLAGS -D_GNU_SOURCE,g" GNUmakefile
make -s KERNEL_DIR=..
cd ..
%endif


###
### install
###

%install

cd linux-%{kversion}

# architectures that don't get kernel-source (i586/i686/athlon) dont need
# much of an install because the build phase already copied the needed files

%if %{buildsource}
mkdir -p $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}

# clean up the source tree so that it is ready for users to build their own
# kernel
make -s mrproper

# copy the source over
tar cf - . | tar xf - -C $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}

# set the EXTRAVERSION to <version>custom, so that people who follow a kernel building howto
# don't accidentally overwrite their currently working moduleset and hose
# their system
perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -%{release}custom/" $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}/Makefile
install -m 644 %{SOURCE10} $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}
%endif

%if %{builddevfs}
# devfs
mkdir -p $RPM_BUILD_ROOT/sbin $RPM_BUILD_ROOT%{_mandir}/man8 $RPM_BUILD_ROOT/etc
install -m 755 -s devfsd/devfsd $RPM_BUILD_ROOT/sbin/devfsd
cp -p devfsd/devfsd.8 $RPM_BUILD_ROOT%{_mandir}/man8
cp -p devfsd/devfsd.conf $RPM_BUILD_ROOT/etc/devfsd.conf
cp -p devfsd/modules.devfs $RPM_BUILD_ROOT/etc/modules.devfs

	%if %{buildsource}
		make -s clean -C $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}/devfsd
		rm -f $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}/devfsd/devfsd
	%endif
%endif


###
### clean
###

%clean
rm -rf $RPM_BUILD_ROOT


###
### scripts
###

# load the loop module for upgrades...in case the old modules get removed we have
# loopback in the kernel so that mkinitrd will work.
%pre 
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
exit 0

%pre smp
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
exit 0

%pre bigmem
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
exit 0

%post 
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --mkinitrd --depmod --install %{KVERREL}
[ -x /usr/sbin/hardlink ] && /usr/sbin/hardlink -c /lib/modules/2.6.* || :

cd /boot
ln -sf vmlinuz-%{KVERREL} vmlinuz
ln -sf System.map-%{KVERREL} System.map
depmod -a -F /boot/System.map-%{KVERREL} %{KVERREL}

%post smp
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --mkinitrd --depmod --install %{KVERREL}smp
[ -x /usr/sbin/hardlink ] && /usr/sbin/hardlink -c /lib/modules/2.6.* || :

%post bigmem
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --mkinitrd --depmod --install %{KVERREL}bigmem
[ -x /usr/sbin/hardlink ] && /usr/sbin/hardlink -c /lib/modules/2.6.* || :


%preun
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}

rm -f /lib/modules/%{KVERREL}/modules.*

%preun smp
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}smp

%preun bigmem
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}bigmem


###
### file lists
###

%if %{buildup}
%files 
%defattr(-,root,root)
##/boot/vmlinu?-%{KVERREL}
/boot/vmlinuz-%{KVERREL}
/boot/System.map-%{KVERREL}
/boot/config-%{KVERREL}
%dir /lib/modules/%{KVERREL}
/lib/modules/%{KVERREL}/kernel
/lib/modules/%{KVERREL}/build

##%files unsupported
##%defattr(-,root,root)
##/lib/modules/%{KVERREL}/unsupported

%endif

%if %{buildsmp}
%files smp
%defattr(-,root,root)
/boot/vmlinu?-%{KVERREL}smp
/boot/System.map-%{KVERREL}smp
/boot/config-%{KVERREL}smp
%dir /lib/modules/%{KVERREL}smp
/lib/modules/%{KVERREL}smp/kernel
/lib/modules/%{KVERREL}smp/build

%files smp-unsupported
%defattr(-,root,root)
/lib/modules/%{KVERREL}smp/unsupported
%endif

%if %{buildbigmem}
%files bigmem
%defattr(-,root,root)
/boot/vmlinu?-%{KVERREL}bigmem
/boot/System.map-%{KVERREL}bigmem
/boot/config-%{KVERREL}bigmem
%dir /lib/modules/%{KVERREL}bigmem
/lib/modules/%{KVERREL}bigmem/kernel
/lib/modules/%{KVERREL}bigmem/build

%files bigmem-unsupported
%defattr(-,root,root)
/lib/modules/%{KVERREL}bigmem/unsupported
%endif

# only some architecture builds need kernel-source and kernel-doc

%if %{buildsource}

%files source
%defattr(-,root,root)
/usr/src/linux-%{KVERREL}/

%files doc
%defattr(-,root,root)
%doc linux-%{kversion}/Documentation/*

%endif

%if %{builddevfs}
%files -n devfsd
%defattr(-,root,root)
/sbin/devfsd
%{_mandir}/man8/devfsd.8*
%config /etc/devfsd.conf
%config /etc/modules.devfs
%endif


###
### change log
###

%changelog
* Sat Dec 20 2003 Yuichiro Nakada <berry@po.yui.mine.nu>
- Create for Berry Linux
- from Red Hat Fedora Core 1
